/*--------------------------Divider_Prime_101.c  -----------------------------------
 * 
 * 
 * Running on ATMega 328p with 8MHz internal clock
 * Hardware: PCB Boards Divider 1.0.1-A / 2024_04
 *                      Clock Divider 1.2.0-B / 2024_07
 * 
 * avrdude -p m328p -c avrisp2                      ; console command
 * avrdude: Device signature = 0x1e950f             ; default output
 * avrdude: safemode: Fuses OK (E:07, H:D9, L:62)   ; default output
 * 
 * 
 * Fuse settings:
 * avrdude -p m328p -c avrisp2 -U lfuse:w:0xe2:m 
 * avrdude -p m328p -c avrisp2 -U hfuse:w:0xd9:m 
 * avrdude -p m328p -c avrisp2 -U efuse:w:0x07:m 
 * 
 * 
 * compile:
 * avr-gcc -Wall -Os -mmcu=atmega328p Divider_Prime_101.c -o Divider_Prime_101.elf
 * avr-objcopy -O ihex -j .text -j .data Divider_Prime_101.elf Divider_Prime_101.hex
 * avrdude -p m328p -c avrisp2 -U flash:w:Divider_Prime_101.hex
 * 
 * 
 * 
 * 21.07.2024
 * adapted from  CLK_Divider_Prim_090.c 05.03.16
 * translated comments
 * removed redundant comments
 * changed formatting
 * new feature software reset not yet implemented
 * Harald
 * ---------------------------------------------------------------------------------------*/

#include <avr/io.h>
#include <avr/interrupt.h>

volatile uint8_t CLK_counter_11;			//  pulse counters
volatile uint8_t CLK_counter_13;
volatile uint8_t CLK_counter_17;
volatile uint8_t CLK_counter_19;
volatile uint8_t CLK_counter_23;
volatile uint8_t CLK_counter_29;
volatile uint8_t CLK_counter_31;


// interrupt INT0

ISR(INT0_vect )                          // input goes high
    
    { 
        CLK_counter_11++;                // increase clock counter on rising edge
        CLK_counter_13++; 
        CLK_counter_17++; 
        CLK_counter_19++; 
        CLK_counter_23++; 
        CLK_counter_29++; 
        CLK_counter_31++; 
        
        PORTB = 0b00000001;               // input LED on
        
        if (CLK_counter_11 == 11)         // Divide by 11
        {
            PORTB = PORTB | 0b00000010;
            CLK_counter_11 = 0;
        }
        
        if (CLK_counter_13 == 13)         // Divide by 13
        {
            PORTB =PORTB | 0b00000100;
            CLK_counter_13 = 0;
        } 
        
        if (CLK_counter_17 == 17)         // Divide by 17
        {
            PORTB =PORTB | 0b00001000;
            CLK_counter_17 = 0;
        } 
        
        if (CLK_counter_19 == 19)         // Divide by 19
        {
            PORTB =PORTB | 0b00010000;
            CLK_counter_19 = 0;
        } 
        
         if (CLK_counter_23 == 23)         // Divide by 23
        {
            PORTB =PORTB | 0b00100000;
            CLK_counter_23 = 0;
        } 
       
         if (CLK_counter_29 == 29)         // Divide by 29
        {
            PORTB =PORTB | 0b01000000;
            CLK_counter_29 = 0;
        } 
        
         if (CLK_counter_31 == 31)         // Divide by 31
        {
            PORTB =PORTB | 0b10000000;
            CLK_counter_31 = 0;
        } 
    } 

ISR(INT1_vect)              // input in goes low
{
    PORTB = 0b00000000;     // All outputs to zero
}
    
    

/* ----------------------MAIN ---------------------------*/
int main (void)
{
  DDRD &=~ (1 << PD3);                   	// set PD3 as input; INT0  
  DDRD &=~ (1 << PD2);                    // set PD2 as input; INT1  
  PORTD |= (1 << PD2);
  DDRB = 0b11111111;                      // PORTB output
  EIMSK |= (1<<INT0);		                 	// enable INT0
  EIMSK |= (1<<INT1);		                 	// enable INT1
  EICRA |= (1<<ISC00)|(1<<ISC01);         // INT0 - Rising edge on INT0 generates an interrupt request
  EICRA |= (1<<ISC11);                    // INT1 - Falling edge of INT1 generates interrupt request
  PORTB = 0b00000000;                     // All outputs to zero, 
  sei();			                            // Interrupts activate
  while (1)                               // for ever
    {								
    }
}
